package it.amattioli.locate;


import java.util.ArrayList;
import java.util.Arrays;

import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.hibernate.validator.Length;

import org.hibernate.validator.NotNull;

import it.amattioli.applicate.commands.AbstractCommand;
import it.amattioli.applicate.commands.Resettable;
import it.amattioli.dominate.util.GenericComparator;
import it.amattioli.dominate.validation.ValidationResult;
import it.amattioli.dominate.validation.ValidationResult.ResultType;


public class PlaceEditor extends AbstractCommand implements Resettable {
	private Place country;
	private Place region;
	private Place province;
	private Place newPlace = new Place();
	private String zipCode;
	private GenericComparator<Place> placeComparator = new GenericComparator<Place>("description", true);
	
	
	public List<Place> getCountryValues() {
		List<Place> result = new ArrayList<Place>();
		PlaceSpecification spec = new PlaceSpecification();
		spec.setPlaceTypes(Arrays.asList(new PlaceType[]{PlaceType.COUNTRY}));
		result.addAll(Place.repository().list(spec));
		Collections.sort(result, placeComparator);
		result.add(0, null);
		return result;
	}
	
	public Place getCountry() {
		return country;
	}

	public void setCountry(Place country) {
		this.country = country;
		firePropertyChange("regionValues", null, null);
		firePropertyChange("provinceValues", null, null);
		firePropertyChange("availableAttribute", null, null);
		firePropertyChange("typeValues", null, null);
	}
	
	public List<Place> getRegionValues() {
		List<Place> result = new ArrayList<Place>();
		if (country != null) {
			PlaceSpecification spec = new PlaceSpecification();
			spec.setPlaceTypes(Arrays.asList(new PlaceType[]{PlaceType.REGION}));
			spec.setParent(country);
			result.addAll(Place.repository().list(spec));
			Collections.sort(result, placeComparator);
		}
		result.add(0, null);
		return result;
	}

	public void setRegion(Place region) {
		this.region = region;
		firePropertyChange("provinceValues", null, null);
		firePropertyChange("availableAttribute", null, null);
		firePropertyChange("typeValues", null, null);
	}

	public Place getRegion() {
		return region;
	}
	
	public List<Place> getProvinceValues() {
		List<Place> result = new ArrayList<Place>();
		Place parentPlace = region;
		if (parentPlace == null) {
			parentPlace = country;
		}
		if (parentPlace != null) {
			PlaceSpecification spec = new PlaceSpecification();
			spec.setPlaceTypes(Arrays.asList(new PlaceType[]{PlaceType.PROVINCE}));
			spec.setParent(parentPlace);
			result.addAll(Place.repository().list(spec));
			Collections.sort(result, placeComparator);
		}
		result.add(0, null);
		return result;
	}
	
	public Place getProvince() {
		return province;
	}

	public void setProvince(Place province) {
		Place old = this.province;
		this.province = province;
		firePropertyChange("province", old, province);
		firePropertyChange("typeValues", null, null);
	}
	
	public void setDescription(String description) {
		this.newPlace.setDescription(StringUtils.upperCase(description));
		firePropertyChange("description", null, null);
	}
	
	public String getDescription() {
		return this.newPlace.getDescription();
	}
	
	public ValidationResult validateDescription(String description) {
		if (StringUtils.isBlank(description)) {
			return new ValidationResult(ResultType.INVALID, "Inserire valore non nullo");
		}
		return null;
	}
	
	public void setShortDescription(String shortDescription) {
		this.newPlace.setShortDescription(StringUtils.upperCase(shortDescription));
	}
	
	@Length(max=2)
	public String getShortDescription() {
		return this.newPlace.getShortDescription();
	}
	
	public void setZipCode(String zipCode) {
		this.zipCode = zipCode;
	}
	
	public String getZipCode() {
		return this.zipCode;
	}
	
	public void setType(PlaceType type) {
		this.newPlace.setType(type);
	}
	
	public ValidationResult validateZipCode(String zipCode) {
		if (!StringUtils.isBlank(zipCode) && zipCode.length() > 10) {
			return new ValidationResult(ResultType.INVALID, "il cap non pu superare 10 caratteri");
		}
		if (PlaceType.TOWN.equals(getType()) && StringUtils.isBlank(zipCode)) {
			return new ValidationResult(ResultType.INVALID, "inserire un cap per il luogo");
		}
		return null;
	}
	
	public List<PlaceType> getTypeValues() {
		List<PlaceType> result = new ArrayList<PlaceType>(Arrays.asList(PlaceType.values()));
		if (country != null) {
			result.remove(PlaceType.COUNTRY);
		} else {
			result.remove(PlaceType.REGION);
			result.remove(PlaceType.PROVINCE);
			result.remove(PlaceType.TOWN);
		}
		if (region != null) {
			result.remove(PlaceType.REGION);
		}
		if (province != null) {
			result.remove(PlaceType.PROVINCE);
			result.remove(PlaceType.REGION);
		}
		return result;
	}
	
	@NotNull
	public PlaceType getType() {
		if (this.newPlace.getType() == null || !getTypeValues().contains(this.newPlace.getType())) {
			this.newPlace.setType(getTypeValues().get(0));
		}
		return this.newPlace.getType();
	}
	
	public boolean isAvailableAttribute(String attribute) {
		if ("region".equals(attribute)) {
			return true;
		} else if ("province".equals(attribute)) {
			return true;
		}
		return false;
	}
	
	public Place getEditingBean() {
		return this.newPlace;
	}
	
	private Place getParentPlace() {
		Place parentPlace = province;
		if (parentPlace == null) {
			parentPlace = region;
		}
		if (parentPlace == null) {
			parentPlace = country;
		}
		return parentPlace;
	}
	
	@Override
	public void doCommand() {
		PlaceSpecification spec = new PlaceSpecification();
		spec.setPlaceTypes(Arrays.asList(new PlaceType[]{getType()}));
		spec.setParent(getParentPlace());
		spec.setDescription(getDescription());
		List<Place> luoghiTrovati = Place.repository().list(spec);
		if (luoghiTrovati.isEmpty()) {
			newPlace.addZipCode(getZipCode());
			newPlace.setParent(getParentPlace());
			Place.repository().put(newPlace);
		} else {
			for (Place luogo:luoghiTrovati) {
				luogo.addZipCode(getZipCode());
				Place.repository().put(luogo);
			}
		}
		
		super.doCommand();
	}

	@Override
	public void cancelCommand() {
		super.cancelCommand();
	}

	@Override
	public void reset() {
		newPlace = new Place();
		country = null;
		region = null;
		province = null;
		zipCode = null;
	}

	@Override
	public boolean toBeReset() {
		return true;
	}
	
	public ValidationResult validate() {
		PlaceSpecification spec = new PlaceSpecification();
		spec.setPlaceTypes(Arrays.asList(new PlaceType[]{getType()}));
		spec.setParent(getParentPlace());
		spec.setDescription(getDescription());
		spec.setZipCode(getZipCode());
		List<Place> luoghiTrovati = Place.repository().list(spec);
		if (!luoghiTrovati.isEmpty()) {
			return new ValidationResult(ResultType.INVALID, "Luogo gi esistente");
		}
		return null;
	}
}
