package it.amattioli.locate;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import it.amattioli.dominate.hibernate.filters.ComparisonType;
import it.amattioli.dominate.specifications.Assembler;
import it.amattioli.dominate.specifications.ConjunctionSpecification;
import it.amattioli.dominate.specifications.EntitySpecification;
import it.amattioli.dominate.specifications.MultipleEnumSpecification;
import it.amattioli.dominate.specifications.StringSpecification;
import it.amattioli.dominate.specifications.hibernate.CriteriaAssembler;
import it.amattioli.locate.hibernate.PlaceTypeHibernateType;

public class PlaceSpecification extends ConjunctionSpecification<Place> {
	private StringSpecification<Place> descriptionSpecification = StringSpecification.newInstance("description");
	private Map<String, StringSpecification<Place>> codeSpecifications = new HashMap<String, StringSpecification<Place>>();
	private MultipleEnumSpecification<Place, PlaceType> placeTypeSpec = MultipleEnumSpecification.newInstance("type", PlaceType.class, PlaceTypeHibernateType.class);
	private EntitySpecification<Place, Long, Place> parentSpecification = EntitySpecification.newInstance("parent", Place.class);
	private CollectionSpecification<Place> zipCodeSpecification = new CollectionSpecification<Place>("zipCodes");
	
	public PlaceSpecification() {
		addSpecification(descriptionSpecification);
		addSpecification(placeTypeSpec);
		addSpecification(parentSpecification);
		addSpecification(zipCodeSpecification);
	}
	
	public String getZipCode() {
		return (String)zipCodeSpecification.getValue();
	}
	
	public void setZipCode(String zipCode) {
		zipCodeSpecification.setValue(zipCode);
	}
	
	public String getDescription() {
		return descriptionSpecification.getValue();
	}
	
	public void setDescription(String description) {
		String old = getDescription();
		descriptionSpecification.setValue(description);
		firePropertyChange("description", old, description);
	}
	
	public ComparisonType getDescriptionType() {
		return descriptionSpecification.getComparisonType();
	}
	
	public void setDescriptionType(ComparisonType filterType) {
		ComparisonType old = getDescriptionType();
		descriptionSpecification.setComparisonType(filterType);
		firePropertyChange("descriptionType", old, filterType);
	}
	
	public Collection<PlaceType> getPlaceTypes() {
		return placeTypeSpec.getValue();
	}

	public void setPlaceTypes(Collection<PlaceType> placeTypes) {
		placeTypeSpec.setValue(placeTypes);
	}
	
	public String getCode(String codeName) {
		StringSpecification<Place> codeSpecification = codeSpecifications.get(codeName);
		if (codeSpecification != null) {
			return codeSpecification.getValue();
		} else {
			return null;
		}
	}
	
	public void setCode(String codeName, String value) {
		StringSpecification<Place> codeSpecification = codeSpecifications.get(codeName);
		if (codeSpecification == null) {
			codeSpecification = StringSpecification.newInstance("code("+codeName+")");
			codeSpecification.setComparisonType(ComparisonType.EXACT);
			codeSpecifications.put(codeName, codeSpecification);
			addSpecification(codeSpecification);
		}
		codeSpecification.setValue(value);
	}

	public void setParent(Place parent) {
		parentSpecification.setValue(parent);
	}
	
	public Place getParent() {
		return parentSpecification.getValue();
	}
	
	@Override
	public boolean supportsAssembler(Assembler assembler) {
		if (assembler instanceof CriteriaAssembler) return false;
		return super.supportsAssembler(assembler);
	}
}
