package it.amattioli.locate.ui.zk;

import static it.amattioli.dominate.validation.ValidationResult.ResultType.INVALID;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.apache.commons.beanutils.NestedNullException;
import org.apache.commons.beanutils.PropertyUtils;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.WrongValueException;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.Events;
import org.zkoss.zk.ui.event.OpenEvent;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Constraint;

import it.amattioli.locate.PlaceSelector;
import it.amattioli.locate.PlaceSelector.Match;
import it.amattioli.dominate.validation.ValidationResult;
import it.amattioli.guidate.containers.BackBeans;
import it.amattioli.guidate.properties.PropertyNameRetriever;
import it.amattioli.guidate.properties.TextPropertyComposer;
import it.amattioli.guidate.validators.PropertyValidator;
import it.amattioli.guidate.validators.ValidatingComposer;

public class PlaceSelectorComposer extends TextPropertyComposer {
	private static final String IS_OPEN_ATTRIBUTE = "it.amattioli.locate.ui.zk.PlaceSelectorComposer.isOpen";
	//private boolean isOpen = false;
	/*
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		Object val = null;
		try {
			Object backBean = getBackBean(comp);
			if (backBean != null) {
				val = PropertyUtils.getProperty(backBean, getCompletePropertyName(comp));
			}
		} catch (NestedNullException e) {
			val = null;
		} catch (NoSuchMethodException e) {
			val = null;
		}
		if (val != null) {
			comp.setAttribute(BackBeans.BACK_BEAN_ATTRIBUTE, val);
			comp.setAttribute(PropertyNameRetriever.PROPERTY_NAME, "description");
			setPropertyClass(String.class);
			setPropertyName("description");
			PlaceSelector selector = (PlaceSelector)val;
			final Bandbox bandbox = (Bandbox)comp;
			bandbox.getDropdown().setAttribute(BackBeans.BACK_BEAN_ATTRIBUTE, selector.getPlaceBrowser());
			selector.addPropertyChangeListener(new PropertyChangeListener() {
	
				@Override
				public void propertyChange(PropertyChangeEvent evt) {
					if ("selectedObject".equals(evt.getPropertyName()) && isOpen) {
						bandbox.close();
					}
				}
				
			});
		}
	}
	*/
	
	public boolean isOpen(Component comp) {
		return (Boolean)comp.getAttribute(IS_OPEN_ATTRIBUTE);
	}
	
	public void setOpen(Component comp, boolean open) {
		comp.setAttribute(IS_OPEN_ATTRIBUTE, open);
	}
	
	public void registerAsValidable(Component comp) throws Exception {
    	if (comp.getParent() != null) {
        	Component container = BackBeans.findContainer(comp.getParent());
    		if (container != null) {
    			Events.sendEvent(new Event(ValidatingComposer.ON_REGISTER_VALIDABLE_ELEMENT, container, comp));
    		}
    	}
    }
	
	public void onCreate(Event evt) throws Exception {
		Component comp = evt.getTarget();
		registerAsValidable(comp);
		setOpen(comp, false);
		Object val = null;
		try {
			Object backBean = getBackBean(comp);
			if (backBean != null) {
				String selectorName = (String)comp.getAttribute("selectorName");
				val = PropertyUtils.getProperty(backBean, selectorName);
			}
		} catch (NestedNullException e) {
			e.printStackTrace();
			val = null;
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			val = null;
		}
		if (val != null) {
			comp.setAttribute(BackBeans.BACK_BEAN_ATTRIBUTE, val);
			comp.setAttribute(PropertyNameRetriever.PROPERTY_NAME, "description");
			setPropertyClass(String.class);
			setPropertyName("description");
			PlaceSelector selector = (PlaceSelector)val;
			final Bandbox bandbox = (Bandbox)comp;
			bandbox.getDropdown().setAttribute(BackBeans.BACK_BEAN_ATTRIBUTE, selector.getPlaceBrowser());
			selector.addPropertyChangeListener(new PropertyChangeListener() {
	
				@Override
				public void propertyChange(PropertyChangeEvent evt) {
					if ("selectedObject".equals(evt.getPropertyName()) && isOpen(bandbox)) {
						bandbox.close();
					}
				}
				
			});
		}
		super.onCreate(evt);
	}
	
	public void onValidateElement(Event evt) {
		PlaceSelector selector = (PlaceSelector)evt.getTarget().getAttribute(BackBeans.BACK_BEAN_ATTRIBUTE);
		if (Match.NONE.equals(selector.getMatch())) {
			throw new WrongValueException(evt.getTarget(), "Luogo sconosciuto");
		}
		ValidationResult result = selector.validate();
		if (result != null && INVALID.equals(result.getType())) {
			throw new WrongValueException(evt.getTarget(), result.getMessage());
		}
    }
	
	@Override
	public void onBlur(Event evt) throws Exception {
		super.onBlur(evt);
		openBandboxIfMultipleMatches((Bandbox)evt.getTarget());
		Events.postEvent("onValidateElement",evt.getTarget(), null);
	}
	
	private void openBandboxIfMultipleMatches(Bandbox bandbox) {
		PlaceSelector selector = (PlaceSelector)bandbox.getAttribute(BackBeans.BACK_BEAN_ATTRIBUTE);
		if (Match.MULTIPLE.equals(selector.getMatch())) {
			bandbox.open();
		}
	}

	public void onOpen(OpenEvent evt) {
		setOpen(evt.getTarget(), evt.isOpen());
		Events.sendEvent(new Event("onOpenBandpopup", ((Bandbox)evt.getTarget()).getDropdown()));
	}

}
